@@ -29,6 +29,8 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
+#include <libxml/xpath.h>
+
 /* Define basic nodoka types */
 typedef void (*ActionInvokedCb)(GtkWindow *nw, const char *key);
 typedef void (*UrlClickedCb)(GtkWindow *nw, const char *url);
@@ -885,16 +887,61 @@ set_notification_text(GtkWindow *nw, const char *summary, const char *body)
 	gtk_label_set_markup(GTK_LABEL(windata->summary_label), str);
 	g_free(str);
 
-	if (pango_parse_markup (body, -1, 0, NULL, NULL, NULL, NULL))
-	{
-		str = g_strdup_printf("<span color=\"#000000\">%s</span>", body);
+	/* body */
+	xmlDocPtr doc;
+	xmlInitParser();
+	str = g_strconcat ("<markup>", "<span color=\"#000000\">", body, "</span>", "</markup>", NULL);
+	/* parse notification body */
+	doc = xmlReadMemory(str, strlen (str), "noname.xml", NULL, 0);
+	g_free (str);
+	if (doc != NULL) {
+		xmlXPathContextPtr xpathCtx;
+		xmlXPathObjectPtr  xpathObj;
+		xmlNodeSetPtr      nodes;
+		const char        *body_label_text;
+		int i, size;
+
+		/* filterout img nodes */
+		xpathCtx = xmlXPathNewContext(doc);
+		xpathObj = xmlXPathEvalExpression((unsigned char *)"//img", xpathCtx);
+		nodes = xpathObj->nodesetval;
+		size = (nodes) ? nodes->nodeNr : 0;
+		for(i = size - 1; i >= 0; i--) {
+			xmlUnlinkNode (nodes->nodeTab[i]);
+			xmlFreeNode (nodes->nodeTab[i]);
+		}
+
+		/* write doc to string */
+		xmlBufferPtr buf = xmlBufferCreate();
+		(void) xmlNodeDump(buf, doc, xmlDocGetRootElement (doc), 0, 0);
+		str = (char *)buf->content;
 		gtk_label_set_markup (GTK_LABEL (windata->body_label), str);
-		g_free(str);
-	}
-	else {
-		gtk_label_set_text (GTK_LABEL (windata->body_label), body);
+
+		/* cleanup */
+		xmlBufferFree (buf);
+		xmlXPathFreeObject (xpathObj);
+		xmlXPathFreeContext (xpathCtx);
+		xmlFreeDoc (doc);
+
+		/* Does it render properly? */
+		body_label_text = gtk_label_get_text (GTK_LABEL (windata->body_label));
+		if ((body_label_text == NULL) || (strlen (body_label_text) == 0)) {
+			goto render_fail;
+		}
+		goto renrer_ok;
 	}
 
+render_fail:
+	/* could not parse notification body */
+	quoted = g_markup_escape_text(body, -1);
+	str = g_strconcat ("<span color=\"#000000\">", quoted, "</span>", NULL);
+	gtk_label_set_markup (GTK_LABEL (windata->body_label), str);
+	g_free (quoted);
+	g_free (str);
+
+renrer_ok:
+	xmlCleanupParser ();
+
 	if (body == NULL || *body == '\0')
 		gtk_widget_hide(windata->body_label);
 	else
